@@ -14,6 +14,8 @@ import sys
 import threading
 import Queue
 
+from mercurial import ui
+
 from tortoisehg.util.i18n import _
 from tortoisehg.util import shlib, hglib
 
@@ -88,8 +90,9 @@ class CmdDialog(gtk.Dialog):
         self.textview.modify_font(pango.FontDescription(fontlog))
         scrolledwindow.add(self.textview)
         self.textbuffer = self.textview.get_buffer()
-        self.textbuffer.create_tag('error', weight=pango.WEIGHT_HEAVY,
-                                   foreground=gtklib.DRED)
+        gtklib.configstyles(ui.ui())
+        for tag, argdict in gtklib.TextBufferTags.iteritems():
+            self.textbuffer.create_tag(tag, **argdict)
 
         self.vbox.pack_start(scrolledwindow, True, True)
         self.connect('map_event', self._on_window_map_event)
@@ -157,24 +160,31 @@ class CmdDialog(gtk.Dialog):
         """
         self.hgthread.process_dialogs()
         enditer = self.textbuffer.get_end_iter()
-        while self.hgthread.getqueue().qsize():
-            try:
-                msg = self.hgthread.getqueue().get(0)
-                self.textbuffer.insert(enditer, hglib.toutf(msg))
-                self.textview.scroll_to_mark(self.textbuffer.get_insert(), 0)
-            except Queue.Empty:
-                pass
         while self.hgthread.geterrqueue().qsize():
             try:
+                tags = gtklib.gettags('ui.error')
                 msg = hglib.toutf(self.hgthread.geterrqueue().get(0))
-                self.textbuffer.insert_with_tags_by_name(enditer, msg, 'error')
+                self.textbuffer.insert_with_tags_by_name(enditer, msg, *tags)
                 self.textview.scroll_to_mark(self.textbuffer.get_insert(), 0)
             except Queue.Empty:
                 pass
         while self.stdoutq.qsize():
             try:
+                tags = gtklib.gettags('ui.error')
                 msg = hglib.toutf(self.stdoutq.get(0))
-                self.textbuffer.insert_with_tags_by_name(enditer, msg, 'error')
+                self.textbuffer.insert_with_tags_by_name(enditer, msg, *tags)
+                self.textview.scroll_to_mark(self.textbuffer.get_insert(), 0)
+            except Queue.Empty:
+                pass
+        while self.hgthread.getqueue().qsize():
+            try:
+                msg, label = self.hgthread.getqueue().get(0)
+                msg = hglib.toutf(msg)
+                tags = gtklib.gettags(label)
+                if tags:
+                    self.textbuffer.insert_with_tags_by_name(enditer, msg, *tags)
+                else:
+                    self.textbuffer.insert(enditer, msg)
                 self.textview.scroll_to_mark(self.textbuffer.get_insert(), 0)
             except Queue.Empty:
                 pass
@@ -257,7 +267,7 @@ class CmdWidget(gtk.VBox):
 
         # tooltips
         if tooltips is None:
-            tooltips = gtk.Tooltips()
+            tooltips = gtklib.Tooltips()
 
         # log viewer
         if self.is_normal:
@@ -277,9 +287,15 @@ class CmdWidget(gtk.VBox):
         else:
             raise _('unknown CmdWidget style: %s') % style
 
+        # progress status frame
+        statframe = gtk.Frame()
+        statframe.set_border_width(4)
+        statframe.set_shadow_type(gtk.SHADOW_NONE)
+        self.pack_start(statframe, False, False)
+
         # progress bar box
         self.progbox = progbox = gtklib.SlimToolbar(tooltips)
-        self.pack_start(progbox)
+        self.pack_start(progbox, False, False)
 
         def add_button(stock_id, tooltip, handler, toggle=False):
             btn = progbox.append_button(stock_id, tooltip, toggle)
@@ -313,6 +329,13 @@ class CmdWidget(gtk.VBox):
             add_icon('succeed', gtk.STOCK_APPLY)
             add_icon('error', gtk.STOCK_DIALOG_ERROR)
 
+            # progres status label
+            self.progstat = gtk.Label()
+            statframe.add(self.progstat)
+            self.progstat.set_alignment(0, 0.5)
+            self.progstat.set_ellipsize(pango.ELLIPSIZE_END)
+            self.progstat.hide()
+
             # progress bar
             self.pbar = gtk.ProgressBar()
             progbox.append_widget(self.pbar, expand=True)
@@ -355,6 +378,7 @@ class CmdWidget(gtk.VBox):
         self.log.clear()
 
         # prepare UI
+        self.inprogress = False
         self.switch_to(working=True)
         self.set_buttons(stop=True, close=False)
         self.already_opened = self.get_pbar()
@@ -406,7 +430,7 @@ class CmdWidget(gtk.VBox):
             markup = markup % (gtklib.DRED, 'bold')
             icons = {'error': True}
         else:
-            markup = markup % ('black', 'normal')
+            markup = markup % (gtklib.NORMAL, 'normal')
             icons = {}
         text = gtklib.markup_escape_text(text)
         self.rlabel.set_markup(markup % text)
@@ -482,12 +506,50 @@ class CmdWidget(gtk.VBox):
 
     ### internal use functions ###
 
+    def clear_progress(self):
+        self.pbar.set_fraction(0)
+        self.pbar.set_text('')
+        self.progstat.set_text('')
+        self.inprogress = False
+
     def update_progress(self):
         if not self.pbar:
             return
         if not self.hgthread.isAlive():
-            self.pbar.set_fraction(0)
-        else:
+            self.clear_progress()
+            return
+
+        data = None
+        while self.hgthread.getprogqueue().qsize():
+            try:
+                data = self.hgthread.getprogqueue().get_nowait()
+            except Queue.Empty:
+                pass
+        counting = False
+        if data:
+            topic, item, pos, total, unit = data
+            if pos is None:
+                self.clear_progress()
+                return
+            if total is None:
+                count = '%d' % pos
+                counting = True
+            else:
+                self.pbar.set_fraction(float(pos) / float(total))
+                count = '%d / %d' % (pos, total)
+            if unit:
+                count += ' ' + unit
+            self.pbar.set_text(count)
+            if item:
+                status = '%s: %s' % (topic, item)
+            else:
+                status = _('Status: %s') % topic
+            self.progstat.set_text(status)
+            if self.progstat.get_property('visible') is False:
+                self.progstat.show()
+            self.inprogress = True
+
+        if not self.inprogress or counting:
             # pulse the progress bar every ~100ms
             tm = shlib.get_system_times()[4]
             if tm - self.last_pbar_update < 0.100:
@@ -500,18 +562,18 @@ class CmdWidget(gtk.VBox):
         self.hgthread.process_dialogs()
 
         # output to buffer
-        while self.hgthread.getqueue().qsize():
-            try:
-                msg = self.hgthread.getqueue().get(0)
-                self.log.append(hglib.toutf(msg))
-            except Queue.Empty:
-                pass
         while self.hgthread.geterrqueue().qsize():
             try:
                 msg = self.hgthread.geterrqueue().get(0)
                 self.log.append(hglib.toutf(msg), error=True)
             except Queue.Empty:
                 pass
+        while self.hgthread.getqueue().qsize():
+            try:
+                msg, label = self.hgthread.getqueue().get(0)
+                self.log.append(hglib.toutf(msg))
+            except Queue.Empty:
+                pass
 
         # update progress bar
         self.update_progress()
@@ -582,8 +644,8 @@ class CmdLogWidget(gtk.VBox):
 
         # text buffer
         self.buffer = self.textview.get_buffer()
-        self.buffer.create_tag('error', weight=pango.WEIGHT_HEAVY,
-                               foreground=gtklib.DRED)
+        for tag, argdict in gtklib.TextBufferTags.iteritems():
+            self.buffer.create_tag(tag, **argdict)
 
     ### public functions ###
 
@@ -596,7 +658,8 @@ class CmdLogWidget(gtk.VBox):
         """
         enditer = self.buffer.get_end_iter()
         if error:
-            self.buffer.insert_with_tags_by_name(enditer, text, 'error')
+            tags = gtklib.gettags('ui.error')
+            self.buffer.insert_with_tags_by_name(enditer, text, *tags)
         else:
             self.buffer.insert(enditer, text)
         self.textview.scroll_to_mark(self.buffer.get_insert(), 0)
@@ -699,7 +762,8 @@ class CmdRunner(object):
         When the command terminates, the callback function is invoked
         with its return code. 
 
-        cmdline: command line string.
+        cmdline: a list of command line arguments or its tuple/list.
+                 All command line arguments must be string, not int or long.
         callback: function invoked after the command terminates.
 
         def callback(returncode, useraborted, ...)
@@ -716,10 +780,13 @@ class CmdRunner(object):
         # clear previous logs
         self.clear_buffers()
 
-        # thread start
-        self.hgthread = hgthread.HgThread(cmdline[1:])
-        self.hgthread.start()
-        gobject.timeout_add(10, self.process_queue, callback, args, kargs)
+        # normalize 'cmdline' arguments
+        if isinstance(cmdline[0], basestring):
+            cmdline = (cmdline,)
+        self.cmdlist = list(cmdline)
+
+        # execute cmdline
+        self.execute_next(callback, args, kargs)
 
         return True
 
@@ -801,6 +868,14 @@ class CmdRunner(object):
         else:
             self.dlg.hide()
 
+    def execute_next(self, callback, args, kargs):
+        if not self.cmdlist:
+            return
+        cmdline = self.cmdlist.pop(0)
+        self.hgthread = hgthread.HgThread(cmdline[1:])
+        self.hgthread.start()
+        gobject.timeout_add(10, self.process_queue, callback, args, kargs)
+
     def process_queue(self, callback, args, kargs):
         # process queue
         self.hgthread.process_dialogs()
@@ -808,7 +883,7 @@ class CmdRunner(object):
         # receive messages from queue
         while self.hgthread.getqueue().qsize():
             try:
-                msg = hglib.toutf(self.hgthread.getqueue().get(0))
+                msg = hglib.toutf(self.hgthread.getqueue().get(0)[0])
                 self.buffer.append((msg, LOG_NORMAL))
                 self.dlg.log.append(msg)
             except Queue.Empty:
@@ -827,9 +902,14 @@ class CmdRunner(object):
             self.hgthread = None
             if len(self.get_err_buffer()) > 0:
                 self.show_log(True)
-            def call_callback():
-                callback(returncode, self.get_buffer(), *args, **kargs)
-            gtklib.idle_add_single_call(call_callback)
+            if returncode == 0 and self.cmdlist:
+                def call_next():
+                    self.execute_next(callback, args, kargs)
+                gtklib.idle_add_single_call(call_next)
+            else:
+                def call_callback():
+                    callback(returncode, self.get_buffer(), *args, **kargs)
+                gtklib.idle_add_single_call(call_callback)
             return False # Stop polling this function
         else:
             return True # Continue polling
