@@ -31,6 +31,8 @@ try:
 except ImportError:
     config_nofork = None
 
+from tortoisehg.hgtk import textview
+
 try:
     import win32con
     openflags = win32con.CREATE_NO_WINDOW
@@ -56,6 +58,9 @@ for sig in ('thg-exit', 'thg-close', 'thg-refresh', 'thg-accept',
 for sig in ('thg-close', 'thg-new'):
     gobject.signal_new(sig, gtk.Notebook,
         gobject.SIGNAL_ACTION, gobject.TYPE_NONE, ())
+for sig in ('thg-undo', 'thg-redo'):
+    gobject.signal_new(sig, textview.UndoableTextView,
+        gobject.SIGNAL_ACTION, gobject.TYPE_NONE, ())
 
 gtkmainalive = False
 def dispatch(args):
@@ -87,10 +92,10 @@ def dispatch(args):
         else:
             gtkrun(run, u, **opts)
 
+origwdir = os.getcwd()
 def portable_fork(ui, opts):
     if 'THG_HGTK_SPAWN' in os.environ or (
-            not opts.get('fork') and (
-                opts.get('nofork') or opts.get('repository'))):
+        not opts.get('fork') and opts.get('nofork')):
         return
     elif ui.configbool('tortoisehg', 'hgtkfork', None) is not None:
         if not ui.configbool('tortoisehg', 'hgtkfork'):
@@ -104,8 +109,8 @@ def portable_fork(ui, opts):
         args = [sys.executable] + sys.argv
     os.environ['THG_HGTK_SPAWN'] = '1'
     cmdline = subprocess.list2cmdline(args)
+    os.chdir(origwdir)
     subprocess.Popen(cmdline,
-                     close_fds=True,
                      creationflags=openflags,
                      shell=True)
     sys.exit(0)
@@ -216,7 +221,6 @@ def _runcatch(ui, args):
     return -1
 
 def runcommand(ui, args):
-    fullargs = args
     cmd, func, args, options, cmdoptions, alias = _parse(ui, args)
     cmdoptions['alias'] = alias
     ui.setconfig("ui", "verbose", str(bool(options["verbose"])))
@@ -252,6 +256,7 @@ def runcommand(ui, args):
     else:
         lui = ui
 
+    hglib.wrapextensionsloader()  # enable blacklist of extensions
     extensions.loadall(ui)
 
     if options['quiet']:
@@ -456,7 +461,7 @@ def forget(ui, *pats, **opts):
 def serve(ui, *pats, **opts):
     """web server"""
     from tortoisehg.hgtk.serve import run
-    if paths.find_root() == None and not opts['webdir_conf']:
+    if paths.find_root() == None and not (opts['web_conf'] or opts['webdir_conf']):
         raise error.RepoError(_("There is no Mercurial repository here"
                     " (.hg not found)"))
     gtkrun(run, ui, *pats, **opts)
@@ -621,7 +626,9 @@ def help_(ui, name=None, with_version=False, **opts):
                 commands = cmds[f].replace("|",", ")
                 ui.write(" %s:\n      %s\n"%(commands, h[f]))
             else:
-                ui.write(' %-*s   %s\n' % (m, f, util.wrap(h[f], m + 4)))
+                ui.write('%s\n' % (util.wrap(h[f],
+                                             initindent=' %-*s   ' % (m, f),
+                                             hangindent=' ' * (m + 4))))
 
         if not ui.quiet:
             addglobalopts(True)
@@ -688,7 +695,11 @@ def help_(ui, name=None, with_version=False, **opts):
         opts_len = max([len(line[0]) for line in opt_output if line[1]] or [0])
         for first, second in opt_output:
             if second:
-                ui.write(" %-*s  %s\n" % (opts_len, first, second))
+                initindent = ' %-*s  ' % (opts_len, first)
+                hangindent = ' ' * (opts_len + 3)
+                ui.write('%s\n' % (util.wrap(second,
+                                             initindent=initindent,
+                                             hangindent=hangindent)))
             else:
                 ui.write("%s\n" % first)
 
@@ -775,7 +786,10 @@ table = {
     "forget": (forget, [], _('hgtk forget [FILE]...')),
     "^serve":
         (serve,
-         [('', 'webdir-conf', '', _('name of the webdir config file'))],
+         [('', 'web-conf', '',
+           _('name of the hgweb config file (serve more than one repository)')),
+          ('', 'webdir-conf', '',
+           _('name of the hgweb config file (DEPRECATED)'))],
          _('hgtk serve [OPTION]...')),
     "thgstatus": (thgstatus,
         [('',  'delay', None, _('wait until the second ticks over')),
@@ -804,7 +818,6 @@ table = {
         [('r', 'rev', '', _('revision to update'))],
         ('hgtk archive')),
     "strip": (strip, [], ('hgtk strip [REV]')),
-    "browse": (browse, [], ('hgtk browse [REV]')),
     "^mpatch": (mpatch, [], ('hgtk mpatch file.rej')),
     "import": (thgimport,
         [('', 'repo', False, _('import to the repository')),
